// -*- C++ -*-
/*!
 * @file  GuiJoystick.cpp
 * @brief Joystick interface module of GUI
 * @date $Date$
 *
 * $Id$
 */

#include "GuiJoystick.h"

#include "interface.h"
#include "support.h"
#include "callbacks.h"

// Module specification
// <rtc-template block="module_spec">
static const char* guijoystick_spec[] =
  {
    "implementation_id", "GuiJoystick",
    "type_name",         "GuiJoystick",
    "description",       "Joystick interface module of GUI",
    "version",           "1.0.2",
    "vendor",            "Tsuyoshi Suenaga, NAIST",
    "category",          "Human Interfaces",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.horizon_gain", "1.0",
    "conf.default.vertical_gain", "1.0",

    ""
  };
// </rtc-template>

GuiJoystick::GuiJoystick(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_VerticalOut("Vertical", m_Vertical),
    m_HorizonOut("Horizon", m_Horizon),
    m_VelocityOut("Velocity", m_Velocity),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("Vertical", m_VerticalOut);
  registerOutPort("Horizon", m_HorizonOut);
  registerOutPort("Velocity", m_VelocityOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  initialize_for_gtk();
}

GuiJoystick::~GuiJoystick()
{
}



RTC::ReturnCode_t GuiJoystick::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("horizon_gain", m_horizon_gain, "1.0");
  bindParameter("vertical_gain", m_vertical_gain, "1.0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GuiJoystick::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GuiJoystick::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GuiJoystick::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t GuiJoystick::onActivated(RTC::UniqueId ec_id)
{
  /// ウィンドウの作成
  window1 = create_window1 ();
  gtk_widget_show (window1);
  /// gtkスタート
  g_thread_create((GThreadFunc)gtk_main_thread, NULL, FALSE, NULL);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t GuiJoystick::onDeactivated(RTC::UniqueId ec_id)
{
  gtk_widget_destroy(window1);
  /// gtk終了
  gtk_main_quit();
  return RTC::RTC_OK;
}



RTC::ReturnCode_t GuiJoystick::onExecute(RTC::UniqueId ec_id)
{
  set_sign(m_horizon_gain, m_vertical_gain);

  /// -1.0〜1.0の値にゲインをかける．
  m_Horizon.data = m_horizon_gain * get_horizon();
  m_Vertical.data = m_vertical_gain * get_vertical();
  m_Velocity.w = m_Horizon.data;
  m_Velocity.v = m_Vertical.data;

  /// データ出力
  m_HorizonOut.write();
  m_VerticalOut.write();
  m_VelocityOut.write();

  std::cout << "(x,y)=(" << m_Horizon.data << "," << m_Vertical.data << ")" << std::endl;

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t GuiJoystick::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GuiJoystick::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GuiJoystick::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GuiJoystick::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GuiJoystick::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void GuiJoystickInit(RTC::Manager* manager)
  {
    RTC::Properties profile(guijoystick_spec);
    manager->registerFactory(profile,
                             RTC::Create<GuiJoystick>,
                             RTC::Delete<GuiJoystick>);
  }
  
};


